
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

;------------------------------------------------------------------------------
;The SendFile function will send a file to the client.
;input:
;    1:ProtocolSession
;    2:ClientSession
;    3:file
;    4:bufferSize
;    5:start (x64: long long, x86:pointer to long long)
;    6:size (x64: long long, x86:pointer to long long)
;    7:timeout
;output:
;    (R/E)AX => true/false
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    SendFile_c:
    StartProc 7

    mov rcx,retSendFile_c
    xchg qword [rsp],rcx
    jmp SendFile

    retSendFile_c:
    sub rsp,7*psize
    EndProc 7
    jmp rcx
else
    SendFile_c:
end if
;---------
;asm implementation
;---------
SendFile:
call denter

mov pebx,pdword [pebp+psize*4] ;(R/E)BX => file
mov pedx,pdword [pebp+psize*5] ;(R/E)DX => bufferSize
;allocate memory to read out the file
push pedx
call GetMemLocation
mov pedi,peax ;(R/E)DI => buffer

if used is32
  ;create long long
  push 0
  push 0

  push pesp ;push long long on stack
end if

push pebx ;file pointer
call getfilesizeLongLong

;test if the file is zero bytes
if used is32
  push pesp ;long long
  call TestZeroQword
else
  mov r12,rax ;R12 => size of file
  test rax,rax
end if

jz endSendFile ;jump if size == 0
mov pecx,pdword [pebp+psize*6] ;ECX = start (x64: long long, x86:pointer to long long)
jpcxz noStartPosition

if used is32
  mov peax,pesp

  push ecx
  push eax
  call SubQwordFromQword   ;qword [esp] => size - start position = size to send
else
  sub r12,rcx              ;r12 => size - start position = size to send
end if
jc errStartSending ;jump if underflow

push pecx ;start position
push pebx ;file
call seekToPointerFile

noStartPosition:
mov pecx,pdword [pebp+psize*7] ;ECX = parameter size to send (x64: long long, x86:pointer to long long)

if used is64
  test pecx,pecx
  cmovz pecx,r12 ;(R/E)CX => size to send (calculated, not from parameter
  jz nosize
else
  jpcxz nosize
end if

if used is32
  mov peax,pesp

  push ecx
  push eax
  call cmpCarryQword   ;is size > filesize
else
  cmp r12,rcx          ;is size > filesize
end if
jc errStartSending ;jump if underflow

if used is32
  add pesp,0x8 ;delete current size from stack
  ;push parameter size on stack
  push dword [pecx+0x4] ;push high dword
  push dword [pecx]     ;push low dword
end if

nosize:
;register status:
;RCX or qword [esp] => size of file to send
;(R/E)DI => buffer
;(R/E)DX => buffer size
;(R/E)BX => file pointer


startSending:
  mov pesi,pedx ;(R/E)SI => buffer size
  if used is32
    cmp dword [pesp+0x4],0
    jnz bufferSizeIsSufficient ;check higher dword on zero

    mov eax,dword [pesp]
    cmp eax,pesi               ;check if the lower dword => buffer size
    jnc bufferSizeIsSufficient ;jump if lower dword => buffer size

    test eax,eax
    jz endStartSending ;jump if the file has been send

    mov pesi,eax ;set the size to send equal to size left to send
  else
    cmp rcx,rsi
    jnc bufferSizeIsSufficient ;jump if lower dword => buffer size

    test rcx,rcx
    jz endStartSending ;stop if the file is send

    mov rsi,rcx
  end if

bufferSizeIsSufficient:
  push pesi ;buffer size
  push pedi ;buffer
  push pebx ;file handle
  call readfile

  push pesi ;push size to send
  mov peax,pesp

  push peax ;&len
  push pedi ;buf
  push pdword [pebp+psize*8]  ;timeout
  push RSF_RETURN_ON_COMPLETE ;flag
  push pdword [pebp+psize*3]  ;sess
  push pdword [pebp+psize*2]  ;psess
  call [SendData]
  test eax,eax
  jnz errStartSending

  push 1
  call sleep

  add pesp,psize ;clear the size value


  if used is64
    sub rcx,rsi ;adjust the size to be send
  else
    mov ecx,esi ;size send
    push pesp ;long long
    call SubToQword ;adjust the size to be send
  end if
jmp startSending

endStartSending:
xor eax,eax
inc eax
jmp endSendFile

errStartSending:
xor eax,eax

endSendFile:
push peax ;save return value

;free buffer
push pedi
call FreeMemLocation

pop peax ;restore return value

call dleave
ret psize*7

;------------------------------------------------------------------------------
;This function will create and send a HTTP-header
;input:
;    1:ProtocolSession
;    2:ClientSession
;    3:responseHeader
;    4:timeout
;output:
;    (R/E)AX => true/false
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    SendHTTPHeader_c:
    StartProc 4

    mov rcx,retSendHTTPHeader_c
    xchg qword [rsp],rcx
    jmp SendHTTPHeader

    retSendHTTPHeader_c:
    sub rsp,4*psize
    EndProc 4
    jmp rcx
else
    SendHTTPHeader_c:
end if
;---------
;asm implementation
;---------
SendHTTPHeader:
call denter
sub pesp,4*2 + psize ;free two pointers on stack (on for size of buffer and one pointer to a buffer

call GetSmallestPortionPageAllocation

mov dword [pebp-UsedSizeBydEnter-4*1],eax   ;Free size on buffer
mov dword [pebp-UsedSizeBydEnter-4*2],eax ;buffer size

push peax
call GetMemLocation
mov pdword [pebp-UsedSizeBydEnter-psize-4*2],peax ;set pointer to buffer

mov pebx,pdword [pebp+psize*4] ;pointer to responseHeader

;send "HTTP/1.1 "
mov pesi,beginRespond
mov ecx,endBeginRespond-beginRespond
call shortSendData
jnz errSendHTTPResponse

mov pesi,Statuscodes
mov pecx,pdword [pebx+4*2] ;get status code
xor eax,eax ;zero eax

compaireStatusCodes:
  lodsw ;get statuscode
  cmp ax,cx
  ;--xor eax,eax ;zero eax
  lodsb ;get size of str
  je foundStatusCode  ;jump if found status code

  xor ah,ah ;zero high word of ax
  add pesi,peax ;increment pointer with str size
  test al,al ;jump if not last item
jne compaireStatusCodes

;statuscode is a pointer
mov pesi,pdword [pebx+4*2] ;get status pointer
push pesi
call strlen          ;calculate the size
mov ecx,eax
jmp calculatedStatusCodeStringSize

foundStatusCode:
xor ecx,ecx
mov cl,al ;ECX = size of statuscode string

calculatedStatusCodeStringSize:
;Send status code with text
call shortSendData
jnz errSendHTTPResponse

;send new line
call shortSendNewLine

test dword [pebx],FRH_DONT_SEND_TIMEDATE
jnz dontSendTimeAndDate

  ;send "Date: "
  mov pesi,dateName
  mov ecx,endDateName-dateName
  call shortSendData
  jnz errSendHTTPResponse

  sub pesp,0x60 ;make space for the 'Date' string

  ;get string with the date
  push pesp
  call GetGMTTime

  ;send date string
  mov pesi,pesp
  push pesi
  call strlen
  mov ecx,eax
  call shortSendData
  jnz errSendHTTPResponse

  add pesp,0x60 ;clear buffer from stack

  ;send new line
  call shortSendNewLine

dontSendTimeAndDate:
mov pesi,connectionStr
mov ecx,endConnectionStr-connectionStr ;length of string 'Connection: '
mov eax,endCloseStr-connectionStr  ;length of string 'Connection: close'

test dword [pebx],FRH_CONNECTION_KEEP_ALIVE
cmovz ecx,eax ;if flag FRH_CONNECTION_KEEP_ALIVE is not set, send string 'Connection: close' instead of 'Connection: '

call shortSendData ;send ', Connection: Close'  or 'Connection: '
jnz errSendHTTPResponse

test dword [pebx],FRH_CONNECTION_KEEP_ALIVE
jz dontSendKeepAliveStr

  mov pesi,keepAliveStr
  mov ecx,endKeepAliveStr-keepAliveStr ;length of string 'Keep-Alive'
  call shortSendData ;send ','Keep-Alive'
  jnz errSendHTTPResponse

dontSendKeepAliveStr:
;send new line
call shortSendNewLine

test dword [pebx],FRH_TRANSFER_ENCODING_CHUNKED
jz dontSendTransferEncodingStr

  mov pesi,transferEncodingStr
  mov ecx,endTransferEncodingStr-transferEncodingStr ;length of string 'Transfer-Encoding: chunked'
  call shortSendData
  jnz errSendHTTPResponse

  ;send new line
  call shortSendNewLine

dontSendTransferEncodingStr:
;send Server string
mov pesi,serverName
mov ecx,endServerName-serverName
call shortSendData

mov pedx,pdword [pebx+4*2+psize*3] ;content-length

test dword [pebx],FRH_FORCE_ZERO_CONTENT_LENGTH  ;force sending content length, even if the field is set to zero
jnz sendContentLength

test pedx,pedx ;if the content-length field is zero, don't send
jz dontSendLength

if used is32
  mov eax,dword [pedx]
  test eax,eax
  jz dontSendLength
end if


sendContentLength:
  ;send new line
  call shortSendNewLine

  ;Send Content-Length field
  mov pesi,contentLengthStr
  mov ecx,endContentLengthStr-contentLengthStr
  call shortSendData
  jnz errSendHTTPResponse

  call ConvIntToStr_And_Send

dontSendLength:

test dword [pebx],FRH_DONT_SEND_CONTENT_TYPE  ;should field Content-Type be represented in the response header?
jnz doneSendingContentType

;send new line
call shortSendNewLine

;send Content-type string
mov pesi,contentType
mov ecx,endContentType-contentType
call shortSendData
jnz errSendHTTPResponse

test dword [pebx],FRH_USE_CONTENT_TYPE_EXTENSION
jz sendConttentTypeThroughContentTypeField

  ;--use content_type_extension to send the contenttype
  mov peax,pdword [pebx+4*2+psize*2]    ;content_type_extension
  call getMimeTypeFromExtension
  test peax,peax
  jz sendConttentTypeThroughContentTypeField

  mov pesi,peax
  jpcxz freeStringAndSendConttentTypeThroughContentTypeField

  dec pecx ;Get rid of the zero byte
  call shortSendData ;send the file type
  jnz errSendHTTPResponse

freeStringAndSendConttentTypeThroughContentTypeField:
  push pesi
  call FreeMemLocation
  jmp doneSendingContentType


sendConttentTypeThroughContentTypeField:

  ;--use content_type to send the contenttype
  mov pecx,pdword [pebx+4*2+psize*1] ;content_type
  mov pesi,contentTypeList
  xor eax,eax ;clear EAX
  lodsb ;read out the size of the string
  jecxz sendContentType ;send the first conttent-type from the list if content_type is zero

  loopThroughContentTypeList:
    add pesi,peax ;add the size of the string to the pointer
    lodsb         ;read out the size of the next string
    test al,al    ;check if this is the last string in the list
    jz endLoopThroughContentTypeList

    dec ecx ;is the index turned zero? send the
  jz sendContentType
  jmp loopThroughContentTypeList


  endLoopThroughContentTypeList:
  ;conttent-type is not a list index, it is a pointer
  mov pesi,pdword [pebx+4*2+psize*1] ;content_type
  push pesi
  call strlen

  sendContentType:
  mov ecx,eax
  call shortSendData ;send the file type
  jnz errSendHTTPResponse


doneSendingContentType:

test dword [pebx],FRH_SEND_CONTENT_RANGE  ;send 'content-range' field
jz dontSendContentRangeField

  ;send new line
  call shortSendNewLine

  ;Send Content-Range field
  mov pesi,contentRangeStr
  mov ecx,endContentRangeStr-contentRangeStr
  call shortSendData
  jnz errSendHTTPResponse

  ;send content_range_offset
  mov pedx,pdword [pebx+4*2+psize*4]
  call ConvIntToStr_And_Send

  ;send '-'
  push '-'
  mov pesi,pesp
  mov ecx,1
  call shortSendData
  jnz errSendHTTPResponse
  add pesp,psize

  ;calculate the max-range (offset + content-length)
  if used is32
    ;copy content_range_offset
    push dword [pedx+4]
    push dword [pedx]

    mov ecx,dword [pebx+4*2+3*psize] ;content length
    push esp                    ;copy of content_range_offset
    call AddQwordToQword

    mov ecx,1
    push esp
    call SubToQword ;subtract by 1

    mov edx,pesp
    call ConvIntToStr_And_Send  ;send max-range
    add pesp,0x8 ;delete copy of content_range_offset
  else
    add rdx,qword [rbx+4*2+3*psize] ;RDX =  content_range_offset + content length
    dec rdx                     ;RDX -= 1
    call ConvIntToStr_And_Send  ;send max-range
  end if

  ;send '/'
  push '/'
  mov pesi,pesp
  mov ecx,1
  call shortSendData
  jnz errSendHTTPResponse
  add pesp,psize

  mov pedx,pdword [pebx+4*2+psize*5] ;content_range_total_length
  call ConvIntToStr_And_Send     ;send content_range_total_length

dontSendContentRangeField:
test dword [pebx],FRH_SEND_LAST_MODIFIED  ;send last modified field with header
jz dontSendLastModifiedField

  mov pedi,pdword [pebx+4*2+psize*6] ;last-modified date in seconds since 1970
  if used is32
    test edi,edi
    jz dontSendLastModifiedField  ;it has to be a pointer in x86
  end if

  ;send new line
  call shortSendNewLine

  ;Send Last-Modified field
  mov pesi,lastModifiedStr
  mov ecx,endLastModifiedStr-lastModifiedStr
  call shortSendData
  jnz errSendHTTPResponse

  call SendDateSince1970AsString

dontSendLastModifiedField:
test dword [pebx],FRH_SEND_EXPIRES  ;send expires field with header
jz dontSendExpireField

  mov pedi,pdword [pebx+4*2+psize*7] ;expires date in seconds since 1970
  if used is32
    test edi,edi
    jz dontSendExpireField  ;it has to be a pointer in x86
  end if

  ;send new line
  call shortSendNewLine

  ;Send Last-Modified field
  mov pesi,expiresStr
  mov ecx,endExpiresStr-expiresStr
  call shortSendData
  jnz errSendHTTPResponse

  call SendDateSince1970AsString

dontSendExpireField:
mov pesi,pdword [pebx+4*2+psize*8] ;first cookie_field structure
test pesi,pesi
jz dontSendCookies

sendCookies:
  ;send new line
  call shortSendNewLine

  ;Send Set-Cookie field
  mov pedx,setCookieStr
  mov ecx,endSetCookieStr-setCookieStr
  call shortSendDataEDX
  jnz errSendHTTPResponse

  lodsp ;get pointer to next cookie_field structure
  mov pedi,peax ;save next cookie_field structure on (R/E)DI

  lodsp ;get flags
  mov ebx,eax ;set flags on EBX

  lodsp ;name of the cookie

  ;send name of the cookie
  push pebp                   ;EBP
  push SKIP_SPECIAL_CHARS     ;special characters are not allowed in the name of the cookie
  push pdword [pebp+psize*5]  ;timeout
  push peax                   ;pointer to the cookie name
  push pdword [pebp+psize*3]  ;ClientSession
  push pdword [pebp+psize*2]  ;ProtocolSession
  call EscapeAndSendValue

  ;send equal sign
  push '='
  mov pedx,pesp
  mov ecx,1
  call shortSendDataEDX
  jnz errSendHTTPResponse

  lodsp ;value of the cookie

  test ebx,CF_DONT_ESCAPE_VALUE ;does the value need to be escaped for special characters before sending?
  jnz sendValueWithoutEscape

  ;send value of the cookie
  push pebp                        ;EBP
  push REPLACE_SPACE_WITH_ADD_SIGN ;special characters are not allowed in the name of the cookie
  push pdword [pebp+psize*5]       ;timeout
  push peax                        ;pointer to the cookie name
  push pdword [pebp+psize*3]       ;ClientSession
  push pdword [pebp+psize*2]       ;ProtocolSession
  call EscapeAndSendValue
  jmp valueOfCookieIsSend

  sendValueWithoutEscape:
  ;send value without escape special characters
  mov pedx,peax
  push pedx
  call strlen ;calculate size of value
  mov pecx,peax
  call shortSendDataEDX ;send data
  jnz errSendHTTPResponse

  valueOfCookieIsSend:
  lodsp  ;pointer to domain string
  test peax,peax
  jz noDomain
    push peax ;save pointer to domain

    ;send ';Domain=' string
    mov pedx,domainStr
    mov ecx,endDomainStr-domainStr
    call shortSendDataEDX
    jnz errSendHTTPResponse

    pop pedx ;restore pointer to domain

    ;send domain string
    push pedx
    call strlen ;calculate teh size of the domain string
    mov pedx,pdword [pesp-psize] ;pointer to domain string
    mov pecx,peax ;size of the domain string
    call shortSendDataEDX
    jnz errSendHTTPResponse

  noDomain:
  lodsp ;pointer to path string
  test peax,peax
  jz pathOfCookieIsSend
    push peax ;save pointer to path

    ;send ';Path=' string
    mov pedx,pathStr
    mov ecx,endPathStr-pathStr
    call shortSendDataEDX
    jnz errSendHTTPResponse

    pop pedx ;restore the pointer to path

    test ebx,CF_DONT_ESCAPE_PATH ;does the value need to be escaped for special characters before sending?
    jnz sendPathWithoutEscape

    ;send path of the cookie
    push pebp                          ;EBP
    push DONT_REPLACE_PATH_CHARACTERS  ;dont replace chars reserved for a path
    push pdword [pebp+psize*5]         ;timeout
    push pedx                          ;pointer to the cookie path
    push pdword [pebp+psize*3]         ;ClientSession
    push pdword [pebp+psize*2]         ;ProtocolSession
    call EscapeAndSendValue
    jmp pathOfCookieIsSend

    sendPathWithoutEscape:
    ;send path without escape special characters
    push pedx
    call strlen ;calculate size of value
    mov pecx,peax
    call shortSendDataEDX ;send path of cookie
    jnz errSendHTTPResponse

  pathOfCookieIsSend:
  lodsp ;pointer to expire date
  test ebx,CF_SEND_EXPIRES_DATE ;do we need to include a expire date?
  jz dontSendExpireDate

    if used is32
      test peax,peax
      jz dontSendExpireDate  ;it has to be a pointer in x86
    end if

    push pedi ;save pointer to next cookie_field

    mov pedi,peax ;(pointer to on x86) expire date

    ;send ';Expires=' string
    mov pedx,expireStr
    mov ecx,endExpireStr-expireStr
    call shortSendDataEDX
    jnz errSendHTTPResponse

    call SendDateSince1970AsString

    pop pedi ;restore pointer to next cookie_field

  dontSendExpireDate:
  test ebx,CF_SECURE ;set option 'SECURE' ?
  jz dontSendSecureOption

    ;send ';Secure' string
    mov pedx,secureStr
    mov ecx,endSecureStr-secureStr
    call shortSendDataEDX
    jnz errSendHTTPResponse


  dontSendSecureOption:
  test ebx,CF_HTTP_ONLY ;set option 'HttpOnly' ?
  jz dontSendHttpOnlyOption

    ;send ';HttpOnly' string
    mov pedx,httpOnlyStr
    mov ecx,endHttpOnlyStr-httpOnlyStr
    call shortSendDataEDX
    jnz errSendHTTPResponse


  dontSendHttpOnlyOption:

  mov pesi,pedi ;the next cookie_field structure
  test pesi,pesi
jnz sendCookies

dontSendCookies:
mov pebx,pdword [pebp+psize*4] ;pointer to responseHeader
test dword [pebx],FRH_SEND_AUTHORIZATION_REQUEST  ;send authorization request
jz dontSendAuthorizationField

  ;send new line
  call shortSendNewLine

  mov pesi,wwwAuthenticateStr  ;"WWW-Authenticate: "
  mov ecx,endwwwAuthenticateStr-wwwAuthenticateStr

  mov edx,dword [pebx+4*2+9*psize] ;authFlags
  test edx,AF_DIGEST_AUTHENTICATION
  jnz sendDigestAuthRequest

    ;challengeBasic ("Basic realm=") is right behind the wwwAuthenticateStr string
    add ecx,endChallengeBasic-challengeBasic
    call shortSendData ;send the basic sceme
    jnz errSendHTTPResponse

    ;send realm
    mov pesi,pdword [pebx+4*2+psize*10] ;realm
    push pesi
    call strlen
    mov ecx,eax
    call shortSendDataWithQuote
    jnz errSendHTTPResponse

    jmp dontSendAuthorizationField

  sendDigestAuthRequest:
    call shortSendData ;send "WWW-Authenticate: "
    jnz errSendHTTPResponse

    mov pesi,challengeDigest  ;"Digest realm="
    mov ecx,endChallengeDigest-challengeDigest
    call shortSendData ;send "Digest realm="
    jnz errSendHTTPResponse

    ;send realm
    mov pesi,pdword [pebx+4*2+10*psize] ;realm
    push pesi
    call strlen
    mov ecx,eax
    call shortSendDataWithQuote
    jnz errSendHTTPResponse

    mov pedi,pdword [pebx+4*2+11*psize] ;domain
    test pedi,pedi
    jz dontSendAuthenticateDomain

      mov pesi,domainAuthStr  ;", domain="
      mov ecx,endDomainAuthStr-domainAuthStr
      call shortSendData ;send ", domain="
      jnz errSendHTTPResponse

      ;send domain
      mov pesi,pedi ;domain
      push pesi
      call strlen
      mov ecx,eax
      call shortSendDataWithQuote
      jnz errSendHTTPResponse

    dontSendAuthenticateDomain:
    mov pedi,pdword [pebx+4*2+12*psize] ;nonce
    test pedi,pedi
    jz dontSendNonce

      mov pesi,nonceStr  ;", domain="
      mov ecx,endNonceStr-nonceStr
      call shortSendData ;send ", domain="
      jnz errSendHTTPResponse

      ;send nonce
      mov pesi,pedi ;nonce
      push pesi
      call strlen
      mov ecx,eax
      call shortSendDataWithQuote
      jnz errSendHTTPResponse

    dontSendNonce:
    mov pedi,pdword [pebx+4*2+13*psize] ;opaque
    test pedi,pedi
    jz dontSendOpaque

      mov pesi,opaqueStr  ;", opaque="
      mov ecx,endOpaqueStr-opaqueStr
      call shortSendData ;send ", opaque="
      jnz errSendHTTPResponse

      ;send opaque
      mov pesi,pedi ;opaque
      push pesi
      call strlen
      mov ecx,eax
      call shortSendDataWithQuote
      jnz errSendHTTPResponse

    dontSendOpaque:
    test edx,AF_STALE_TRUE or AF_STALE_FALSE
    jz dontSendStale

      mov pesi,staleStr
      mov ecx,endStaleStr-staleStr ;length of string ', stale='
      mov eax,endTrueStr-staleStr  ;length of string ', stale="True"'

      test edx,AF_STALE_TRUE
      cmovnz ecx,eax ;if flag AF_STALE_TRUE is set, send string ', stale="True"' instead of ', stale='

      call shortSendData ;send ', stale=' or ', stale="True"'
      jnz errSendHTTPResponse

      test edx,AF_STALE_FALSE
      jz dontSendStale

      mov pesi,falseStr
      mov ecx,endFalseStr-falseStr
      call shortSendData ;send '"false"'
      jnz errSendHTTPResponse

    dontSendStale:
    mov pedi,pdword [pebx+4*2+14*psize] ;algorithm
    test pedi,pedi
    jz dontSendAlgorithm

      mov pesi,algorithmStr  ;', algorithm='
      mov ecx,endAlgorithmStr-algorithmStr
      call shortSendData ;send ', algorithm='
      jnz errSendHTTPResponse

      ;send algorithm
      mov pesi,pedi ;algorithm
      push pesi
      call strlen
      mov ecx,eax
      call shortSendDataWithQuote
      jnz errSendHTTPResponse

    dontSendAlgorithm:
    mov pesi,qopStr
    mov ecx,endQopStr-qopStr ;length of string ', qop='
    mov eax,endQopAuthStr-qopStr  ;length of string ', qop="auth"'

    test edx,AF_QOP_AUTH_INT
    cmovz ecx,eax ;if flag AF_QOP_AUTH_INT is not set, send string ', qop="auth"' instead of , qop='

    call shortSendData ;send ', qop="auth"' or , qop='
    jnz errSendHTTPResponse

    test edx,AF_QOP_AUTH_INT
    jz doneSendingQop

      mov pesi,qopAuthIntStr
      mov ecx,endQopAuthIntStr-qopAuthIntStr
      call shortSendData ;send 'auth-int'
      jnz errSendHTTPResponse

    doneSendingQop:

dontSendAuthorizationField:
mov pebx,pdword [pebp+psize*4] ;pointer to responseHeader
mov pedi,pdword [pebx+4*2+(9*psize)+(6*psize)] ;(R/E)DI => member location
test pedi,pedi
jz dontSendLocation

  ;send new line
  call shortSendNewLine

  mov pesi,locationStr  ;'Location: '
  mov ecx,endLocationStr-locationStr
  call shortSendData ;send 'Location: '

  test dword [pebx],FRH_DONT_ESCAPE_LOCATION  ;donbt escape special characters
  jz escapeCharactersForSending

    mov pesi,pedi ;(R/E)SI => member location
    push pesi
    call strlen
    mov pecx,peax
    call shortSendData ;send member location
    jmp sendLocation


  escapeCharactersForSending:
    ;send value of location
    push pebp                         ;EBP
    push DONT_REPLACE_PATH_CHARACTERS ;special characters are not allowed in the name of the cookie
    push pdword [pebp+psize*5]        ;timeout
    push pedi                         ;pointer to the location value
    push pdword [pebp+psize*3]        ;ClientSession
    push pdword [pebp+psize*2]        ;ProtocolSession
    call EscapeAndSendValue

  sendLocation:

dontSendLocation:
;send new line
call shortSendNewLine

test dword [pebx],FRH_DONT_END_HEADER
jnz dontSendTerminatingNewLine

  ;send new line that indicates the end of the header
  call shortSendNewLine

dontSendTerminatingNewLine:
xor eax,eax
inc eax

jmp endSendHTTPResponse

errSendHTTPResponse:
xor eax,eax

endSendHTTPResponse:
call FlushBuffer
;FreeBuffer
push pdword [pebp-UsedSizeBydEnter-psize-4*2]
call FreeMemLocation

call dleave
ret psize*4

;------------------------------------------------------------------------------
;This function will send a new line
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendNewLine:
push pecx
push pesi

mov pesi,newLine
mov pecx,2
call shortSendData

pop pesi
pop pecx
ret

;------------------------------------------------------------------------------
;This function will call shortSendData, but will use DX as input istead of SI
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)DX => buffer
;    (R/E)CX => size
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendDataEDX:
push pesi

mov pesi,pedx
call shortSendData

pop pesi
ret

;---------------------
;Same as FlushBuffer  but sets EBX to 1 to send chunks
FlushBufferWithChunk:
push pebx
xor ebx,ebx
inc ebx
jmp EBXSetFlushBuffer

;------------------------------------------------------------------------------
;This function will send the in the buffer remaining bytes to the client
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    ebp+psize*5:timeout
;    pebp-UsedSizeBydEnter-4 => Free size in buffer
;    pebp-UsedSizeBydEnter-4*2 => buffer size
;    pebp-UsedSizeBydEnter-psize-4*2 => pointer to buffer
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
FlushBuffer:
push pebx
xor ebx,ebx

EBXSetFlushBuffer:
push pesi
push pecx

mov eax,dword [pebp-UsedSizeBydEnter-4*2]   ;buffer size
sub eax,dword [pebp-UsedSizeBydEnter-4]     ;buffer size - Free size on buffer = size currently in use
jz endFlushBuffer

mov pesi,pdword [pebp-UsedSizeBydEnter-psize-4*2] ;pointer to buffer
sub pesi,peax                                     ;get beginning of the buffer

mov pdword [pebp-UsedSizeBydEnter-psize-4*2],pesi ;reset buffer pointer

  push peax ;push len

  test ebx,ebx
  jz donSendChunkSize2

    mov pecx,peax
    call SendSizeChunk

  donSendChunkSize2:

  push pesp                   ;len
  push pesi                   ;buf
  push pdword [pebp+psize*5]  ;timeout
  push RSF_RETURN_ON_COMPLETE ;flags
  push pdword [pebp+psize*3]  ;ClientSession
  push pdword [pebp+psize*2]  ;ProtocolSession
  call [SendData]

add pesp,psize

endFlushBuffer:
pop pecx
pop pesi
pop pebx
test peax,peax
ret

;------------------------------------------------------------------------------
;This function will send chunk size
;input:
;    (R/E)CX Size Chunk
;    [ebp+psize*2]:ProtocolSession
;    [ebp+psize*3]:ClientSession
;    [ebp+psize*5]:timeout
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
SendSizeChunk:
push pedi
push pecx

sub pesp,0x20
mov pedi,pesp

;begin chunk with new line
mov byte [pedi],0x0D
mov byte [pedi+1],0x0A
add pedi,2

;next convert chunk size to string
push 0x10                 ;base at 16
push pedi                 ;output string
push pecx                 ;value
call IntToStr

;add another new line (for the data
push pedi
call strlen
mov byte [pedi+peax],0x0D
mov byte [pedi+peax+1],0x0A

sub pedi,2 ;set pointer to first new line
add peax,4 ;add lines to len

push peax

push pesp ;len
push pedi ;buf
push pdword [ebp+psize*5]    ;timeout
push RSF_RETURN_ON_COMPLETE  ;flags
push pdword [pebp+psize*3] ;ClientSession
push pdword [pebp+psize*2] ;ProtocolSession
call [SendData]

add pesp,0x20+psize ;buff with the len parameter

;restore registers
pop pecx
pop pedi
ret
;---------------------
;Same as shortSendData  but sets EBX to 1 to seend chunks
shortSendDataWithChunk:
push pebx
xor ebx,ebx
inc ebx
jmp EBX_set_shortSendData


;------------------------------------------------------------------------------
;This function will send data
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    ebp+psize*5:timeout
;    (R/E)SI => buffer
;    (R/E)CX => size
;    pebp-UsedSizeBydEnter-4 => Free size in buffer
;    pebp-UsedSizeBydEnter-4*2 => buffer size
;    pebp-UsedSizeBydEnter-psize-4*2 => pointer to buffer
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendData:
push pebx
xor pebx,pebx

EBX_set_shortSendData:
push pedi
push pesi
push pecx

push pesi
push pecx

mov pedi,pdword [pebp-UsedSizeBydEnter-psize-4*2] ;pointer to buffer
mov eax,dword [pebp-UsedSizeBydEnter-4] ;Free size in buffer

;if size>free size then size=free size
cmp ecx,eax
cmova ecx,eax

sub pdword [pesp],pecx                       ;size to send - size set in buffer
sub dword [pebp-UsedSizeBydEnter-4],ecx      ;Free size in buffer -= size set in buffer
add pdword [pesp+psize],pecx                 ;buff + size set in buffer = pointer to buf that is not placed in buffer

rep movsb ;copy bytes to buffer

mov pdword [pebp-UsedSizeBydEnter-psize-4*2],pedi ;set buffer pointer

;check if buffer is full
cmp dword [pebp-UsedSizeBydEnter-4],0 ;Free size == 0 ?
jne buffIsNotFull

EmptyBuffer:
;send and empty buffer
mov pesi,pdword [pebp-UsedSizeBydEnter-psize-4*2] ;pointer to buffer
mov ecx,dword [pebp-UsedSizeBydEnter-4*2]   ;buffer size
sub pesi,pecx ;get beginning of the pointer

mov pdword [pebp-UsedSizeBydEnter-psize-4*2],pesi ;reset buffer pointer
mov dword [pebp-UsedSizeBydEnter-4],ecx     ;reset free size in buffer

  push pecx ;size of buffer
  test ebx,ebx
  jz donSendChunkSize

    call SendSizeChunk

  donSendChunkSize:
;ResendData:
  push pesp                    ;len
  push pesi                    ;buf
  push pdword [pebp+psize*5]   ;timeout
  push RSF_RETURN_ON_COMPLETE  ;flags
  push pdword [pebp+psize*3]   ;ClientSession
  push pdword [pebp+psize*2]   ;ProtocolSession
  call [SendData]

  add pesp,psize

;  sub pecx,pdword [pesp]
;  mov pdword [pesp],pecx ;reset len

;  cmp eax,CE_NO_BUFFER_AVAILABLE
;  je ResendData
;  cmp eax,CE_UNKNOWN_ERROR
;  je ResendData

pop pecx ;get rest of size to set in buffer
pop pesi ;get pointer to bufer
jecxz endShortSendData

call shortSendData

jmp endShortSendData

buffIsNotFull:
pop pecx
pop pesi

endShortSendData:
pop pecx
pop pesi
pop pedi
pop pebx
;add pesp,psize
xor eax,eax
;test eax,eax
ret

;------------------------------------------------------------------------------
;This function will send data with quotes (one in the beginning, one at the end)
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)SI => buffer
;    (R/E)CX => size   '
;    pebp-UsedSizeBydEnter-4 => Free size in buffer
;    pebp-UsedSizeBydEnter-4*2 => buffer size
;    pebp-UsedSizeBydEnter-psize-4*2 => pointer to buffer
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;
;corrupts: (R/E)SI, (R/E)CX
;------------------------------------------------------------------------------
shortSendDataWithQuote:
push '"'
mov peax,pesp

push pesi
push pecx

;send beginning quote
mov pesi,peax
mov ecx,1
call shortSendData

;send string
pop pecx
pop pesi
call shortSendData

mov pesi,pesp
mov ecx,1
call shortSendData

;swap en pop instead of "add pesp,psize" to keep the flags intact.
xchg peax,pdword [pesp]
pop peax

ret

;------------------------------------------------------------------------------
;This function will oonvert a long long with seconds since 1970 to a readable string and send this string
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)DI => qword with seconds from 1970 (pointer in x86)
;    pebp-UsedSizeBydEnter-4 => Free size in buffer
;    pebp-UsedSizeBydEnter-4*2 => buffer size
;    pebp-UsedSizeBydEnter-psize-4*2 => pointer to buffer
;output:
;    none
;corrupt:
;    (R/E)AX, (R/E)DI
;------------------------------------------------------------------------------
SendDateSince1970AsString:
push pesi ;save (R/E)SI
push pecx ;save (R/E)CX

sub pesp,0x18 ;create space for the DateTime structure
;convert seconds since 1970 to a DateTime structure
push pesp ;output DateTime structure
push pedi ;qword with seconds from 1970
call [RecoverDateTime]

mov pedi,pesp ;pointer to DateTime structure
sub pesp,0x60 ;make space for the 'Date' string

;convert DateTime structure to a readable string
push pesp ;output string
push pedi
call SystemTimeToString

;send string date
mov pesi,pesp
push pesi
call strlen
mov ecx,eax
call shortSendData

add pesp,0x60 + 0x18

pop pecx ;restore (R/E)CX
pop pesi ;restore (R/E)SI
ret

;------------------------------------------------------------------------------
;This function will convert qword (or pointer to qword x86) to string
;and send by function SendData
;input:
;    (R/E)DX:qword (or pointer to qword, x86)
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    pebp-UsedSizeBydEnter-4 => Free size in buffer
;    pebp-UsedSizeBydEnter-4*2 => buffer size
;    pebp-UsedSizeBydEnter-psize-4*2 => pointer to buffer
;output:
;    (R/E)AX => SendData return value
;------------------------------------------------------------------------------
ConvIntToStr_And_Send:
push pesi ;save (R/E)SI
push pecx ;save (R/E)CX

sub pesp,0x18 ;free 20 + 1 char for the int to string convertion (algined is 24)
mov pesi,pesp

;convert qword (or pointer to qword x86) to a string
if used is32
  test pedx,pedx
  jnz convertQwordPointer

  ;no pointer was given, set '0'
  mov word [pesi],0x0030

  convertQwordPointer:
  push 10                    ;base
  push pesi                  ;output buffer
  push pedx                  ;pointer to long long
  call IntQToStr
else
  push 10                    ;base
  push pesi                  ;output buffer
  push pedx                  ;qword
  call IntToStr
end if

;send the length
push pesi
call strlen
mov ecx,eax
call shortSendData

add pesp,0x18
pop pecx ;restore (R/E)CX
pop pesi ;restore (R/E)SI
ret

;------------------------------------------------------------------------------
;This function will escape special characters in value and send this value
;input:
;    1:ProtocolSession
;    2:ClientSession
;    3:value to escape and send
;    4:timeout
;    5:flags
;      0x1: REPLACE_SPACE_WITH_ADD_SIGN
;      0x2: SKIP_SPECIAL_CHARS (will not send special chars)
;      0x4: DONT_REPLACE_PATH_CHARACTERS

;output:
;    (R/E)AX => SendData return value
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    EscapeAndSendValue_c:
    StartProc 5

    mov rcx,retEscapeAndSendValue_c
    xchg qword [rsp],rcx
    jmp EscapeAndSendValue_Export

    retEscapeAndSendValue_c:
    sub rsp,5*psize
    EndProc 5
    jmp rcx
else
    EscapeAndSendValue_c:
end if
;---------
;asm implementation
;---------
EscapeAndSendValue_Export:
call denter

sub pesp,4*2 + psize ;free two pointers on stack (on for size of buffer and one pointer to a buffer

call GetSmallestPortionPageAllocation

mov dword [pebp-UsedSizeBydEnter-4*1],eax   ;Free size on buffer
mov dword [pebp-UsedSizeBydEnter-4*2],eax ;buffer size

push peax
call GetMemLocation
mov pdword [pebp-UsedSizeBydEnter-psize-4*2],peax ;set pointer to buffer

push pebp ;(R/E)BB (pointers to buffer
push pdword [pebp+psize*6] ;flags
push pdword [pebp+psize*5] ;timeout
push pdword [pebp+psize*4] ;value to escape and send
push pdword [pebp+psize*3] ;ClientSession
push pdword [pebp+psize*2] ;ProtocolSession
call EscapeAndSendValue

call FlushBuffer
;FreeBuffer
push pdword [pebp-UsedSizeBydEnter-psize-4*2]
call FreeMemLocation

call dleave
ret psize*5

;------------------------------------------------------------------------------
;This function will escape special characters in value and send this value
;input:
;    1:ProtocolSession
;    2:ClientSession
;    3:value to escape and send
;    4:timeout
;    5:flags
;      0x1: REPLACE_SPACE_WITH_ADD_SIGN
;      0x2: SKIP_SPECIAL_CHARS (will not send special chars)
;      0x4: DONT_REPLACE_PATH_CHARACTERS
;    6:pointer to EBP with the following values
;        pebp-UsedSizeBydEnter-4 => Free size in buffer
;        pebp-UsedSizeBydEnter-4*2 => buffer size
;        pebp-UsedSizeBydEnter-psize-4*2 => pointer to buffer

;output:
;    (R/E)AX => SendData return value
;------------------------------------------------------------------------------
EscapeAndSendValue:
call denter

sub pesp,4*2 + psize ;free two pointers on stack (on for size of buffer and one pointer to a buffer

mov peax,pdword [pebp+psize*7] ;pointer to EBP with buffer

mov ecx,dword [peax-UsedSizeBydEnter-4]   ;Free size in buffer
mov dword [pebp-UsedSizeBydEnter-4],ecx   ;Free size on buffer

mov ecx,dword [peax-UsedSizeBydEnter-4*2] ;buffer size
mov dword [pebp-UsedSizeBydEnter-4*2],ecx ;buffer size

mov pecx,pdword [peax-UsedSizeBydEnter-psize-4*2]    ;buffer size
mov pdword [pebp-UsedSizeBydEnter-psize-4*2],pecx    ;buffer

mov pesi,pdword [pebp+psize*4] ;pointer to cookie value
mov pedi,pesi                  ;last pointer of cookie value (not send yet)
xor edx,edx                    ;unsend data pointer (R/E)DI
xor eax,eax                    ;clear EAX

push pesi
call strlen
mov pecx,peax                   ;ECX = size of cookie value
test pecx,pecx
jz EndEscapeAndSendValue

loopThroughCookieValue:
  lodsb

  cmp al,0x21
  jc SendUnsendDataAndEscapeChar  ;if a<21, the char should be escaped
  je nextloopThroughCookieValue   ;if a=21, the char should not be escaped

  cmp al,0x27   ;if a!=21 & a<27, the char should be escaped
  jc SendUnsendDataAndEscapeChar

  cmp al,0x2B   ;if a>27 & a=<2A, the char should not be escaped
  jc nextloopThroughCookieValue

  cmp al,0x2D   ;if a>2A & a<2D, the char should be escaped
  jc SendUnsendDataAndEscapeChar

  cmp al,0x2F   ;if a=>2D & a=<2E, the char should not be escaped
  jc nextloopThroughCookieValue

  cmp al,0x30   ;if a>2F & a<30, the char should be escaped
  jc SendUnsendDataAndEscapeChar

  cmp al,0x3A   ;if a=>30 & a=<39, the char should not be escaped
  jc nextloopThroughCookieValue

  cmp al,0x41   ;if a>39 & a<41, the char should be escaped
  jc SendUnsendDataAndEscapeChar

  cmp al,0x5B   ;if a=>41 & a=<5A, the char should not be escaped
  jc nextloopThroughCookieValue

  cmp al,0x5F   ;if a>5A & a<5F, the char should be escaped
  jc SendUnsendDataAndEscapeChar
  je nextloopThroughCookieValue ;if a==5F do not escape

  cmp al,0x61   ;if a>5F & a<61, the char should be escaped
  jc SendUnsendDataAndEscapeChar

  cmp al,0x7B   ;if a=>61 & a=<7A, the char should not be escaped
  jc nextloopThroughCookieValue

  cmp al,0x7E   ;if a>7A & a<7E, the char should be escaped
  jc SendUnsendDataAndEscapeChar
  je nextloopThroughCookieValue ;if a==7E do not escape

  jmp nextloopThroughCookieValue ;the char should not be escaped

SendUnsendDataAndEscapeChar:
  test pdword [pebp+psize*6],DONT_REPLACE_PATH_CHARACTERS
  jz dontCheckOnPathSymbols

    cmp al,0x24 ;'$'
    je nextloopThroughCookieValue

    cmp al,0x26 ;'&'
    je nextloopThroughCookieValue

    cmp al,0x2B ;'+'
    je nextloopThroughCookieValue

    cmp al,0x2C ;','
    je nextloopThroughCookieValue

    cmp al,0x2F ;'/'
    je nextloopThroughCookieValue

    cmp al,0x3A ;':'
    je nextloopThroughCookieValue

    cmp al,0x3D ;'='
    je nextloopThroughCookieValue

    cmp al,0x40 ;'@'
    je nextloopThroughCookieValue

  dontCheckOnPathSymbols:
  xchg pedi,pesi ;swap current pos ((R/E)SI) with pointer unsend data ((R/E)DI)
  xchg pecx,pedx ;swap counter with size of unsend bytes
  push peax ;save byte to escape

  call shortSendData ;send unsend data

  pop peax ;restore byte to escape

  test pdword [pebp+psize*6],SKIP_SPECIAL_CHARS
  jnz doNextloopThroughCookieValue

  cmp al,' '
  jne isNotSpaceToEscape

  test pdword [pebp+psize*6], REPLACE_SPACE_WITH_ADD_SIGN
  jz isNotSpaceToEscape

  push '+' ;replace space with a add sign
  mov pesi,pesp
  mov ecx,1 ;send one chars
  jmp sendAddSign

  isNotSpaceToEscape:
  push '%' ;push % character
  mov pesi,pesp
  inc pesi ;skip the % char

  push 0x10 ;base 16
  push pesi ;output buffer
  push peax ;number to convert
  call IntToStr

  dec pesi
  mov ecx,3 ;send three chars
  sendAddSign:
  call shortSendData ;send the escaped char
  add pesp,psize

doNextloopThroughCookieValue:
  xor ecx,ecx   ;zero unsend data
  xchg ecx,edx  ;restore counter
  mov pesi,pedi ;restore current pos

  dec pecx
  jnz loopThroughCookieValue
  ;loop loopThroughCookieValue
  jmp endloopThroughCookieValue

nextloopThroughCookieValue:
  inc edx ;one byte unsend
dec pecx
jnz loopThroughCookieValue
;loop loopThroughCookieValue

endloopThroughCookieValue:
;send unsend data
mov pesi,pedi ;(R/E)SI =>with pointer unsend data
mov pecx,pedx ;(R/E)CX => size of unsend data

call shortSendData ;send unsend data

mov peax,pdword [pebp+psize*7] ;pointer to EBP with buffer

mov ecx,dword [pebp-UsedSizeBydEnter-4]   ;Free size in buffer
mov dword [peax-UsedSizeBydEnter-4],ecx   ;Free size on buffer

mov ecx,dword [pebp-UsedSizeBydEnter-4*2] ;buffer size
mov dword [peax-UsedSizeBydEnter-4*2],ecx ;buffer size

mov pecx,pdword [pebp-UsedSizeBydEnter-psize-4*2]    ;buffer size
mov pdword [peax-UsedSizeBydEnter-psize-4*2],pecx    ;buffer

EndEscapeAndSendValue:
call dleave
ret psize*6

;------------------------------------------------------------------------------
;This function will escape special HTML characters in value and send this value
;input:
;    1:ProtocolSession
;    2:ClientSession
;    3:value to escape and send
;    4:timeout
;    5:flags
;      0x1: REPLACE_SPACE_WITH_NON_BREAKING_SPACE
;      0x2: EHSV_SEND_IN_CHUNKS

;output:
;    (R/E)AX => SendData return value
;------------------------------------------------------------------------------
;---------
;C implementation
;---------
if used is64
    EscapeHtmlAndSendValue_c:
    StartProc 5

    mov rcx,retEscapeHtmlAndSendValue_c
    xchg qword [rsp],rcx
    jmp EscapeHtmlAndSendValue_Export

    retEscapeHtmlAndSendValue_c:
    sub rsp,5*psize
    EndProc 5
    jmp rcx
else
    EscapeHtmlAndSendValue_c:
end if
;---------
;asm implementation
;---------
EscapeHtmlAndSendValue_Export:
call denter

sub pesp,4*2 + psize ;free two pointers on stack (on for size of buffer and one pointer to a buffer

call GetSmallestPortionPageAllocation

mov dword [pebp-UsedSizeBydEnter-4*1],eax   ;Free size on buffer
mov dword [pebp-UsedSizeBydEnter-4*2],eax ;buffer size

push peax
call GetMemLocation
mov pdword [pebp-UsedSizeBydEnter-psize-4*2],peax ;set pointer to buffer

if used is32
  mov peax,shortSendData
  test pdword [pebp+psize*6],EHSV_SEND_IN_CHUNKS
  jz dontSendInChunks

    mov peax,shortSendDataWithChunk

  dontSendInChunks:
  push peax
else
  mov r12,shortSendData
  test pdword [pebp+psize*6],EHSV_SEND_IN_CHUNKS
  jz dontSendInChunks

    mov r12,shortSendDataWithChunk

  dontSendInChunks:
end if
push pebp ;(R/E)BB (pointers to buffer
push pdword [pebp+psize*6] ;flags
push pdword [pebp+psize*5] ;timeout
push pdword [pebp+psize*4] ;value to escape and send
push pdword [pebp+psize*3] ;ClientSession
push pdword [pebp+psize*2] ;ProtocolSession
call EscapeHtmlAndSendValue

test pdword [pebp+psize*6],EHSV_SEND_IN_CHUNKS
jz dontFlushInChunks

  call FlushBufferWithChunk
  jmp flushedBuffer

dontFlushInChunks:
  call FlushBuffer

flushedBuffer:
;FreeBuffer
push pdword [pebp-UsedSizeBydEnter-psize-4*2]
call FreeMemLocation

call dleave
ret psize*5

;----------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;This function will escape special HTML characters in value and send this value
;input:
;    1:ProtocolSession
;    2:ClientSession
;    3:value to escape and send
;    4:timeout
;    5:flags
;      0x1: REPLACE_SPACE_WITH_NON_BREAKING_SPACE
;    6:pointer to EBP with the following values
;        pebp-UsedSizeBydEnter-4 => Free size in buffer
;        pebp-UsedSizeBydEnter-4*2 => buffer size
;        pebp-UsedSizeBydEnter-psize-4*2 => pointer to buffer
;in x86: 7: Pointer to shortSendDataWithChunk or shortSendData
;in x64: R12: Pointer to shortSendDataWithChunk or shortSendData

;output:
;    (R/E)AX => SendData return value
;------------------------------------------------------------------------------
EscapeHtmlAndSendValue:
call denter

sub pesp,4*2 + psize ;free two pointers on stack (on for size of buffer and one pointer to a buffer

mov peax,pdword [pebp+psize*7] ;pointer to EBP with buffer

mov ecx,dword [peax-UsedSizeBydEnter-4]   ;Free size in buffer
mov dword [pebp-UsedSizeBydEnter-4],ecx   ;Free size on buffer

mov ecx,dword [peax-UsedSizeBydEnter-4*2] ;buffer size
mov dword [pebp-UsedSizeBydEnter-4*2],ecx ;buffer size

mov pecx,pdword [peax-UsedSizeBydEnter-psize-4*2]    ;buffer size
mov pdword [pebp-UsedSizeBydEnter-psize-4*2],pecx    ;buffer

mov pesi,pdword [pebp+psize*4] ;pointer to cookie value
mov pedi,pesi                  ;last pointer of cookie value (not send yet)
xor edx,edx                    ;unsend data pointer (R/E)DX
xor eax,eax                    ;clear EAX

push pesi
call strlen
mov pecx,peax                   ;ECX = size of value
test pecx,pecx
jz EndEscapeHtmlAndSendValue

loopThroughValue:
  lodsb

  mov pebx,HtmlSpecialCharacters ;List with special, to escape, html characters
  loopHtmlCharacters:
    mov ah,byte [pebx]
    inc pebx

    test ah,ah
    jz nextloopThroughValue ;dontEscape

    ;since the list is numeric-order, you can say that the character doen not have to be escaped if below current special character
    cmp al,ah
    jb nextloopThroughValue
  jne loopHtmlCharacters

  cmp al,0x20
  jne charIsNotSpace

  test pdword [pebp+psize*6],REPLACE_SPACE_WITH_NON_BREAKING_SPACE
  jz nextloopThroughValue ;Dont escape space if flag is not set

charIsNotSpace:
  dec pebx

  push pecx ;save ECX

  ;ECX => Index of the special Html-character
  mov pecx,pebx
  sub pecx,HtmlSpecialCharacters
  mov pebx,HtmlEscapedNames ;(R/E)BX => list with strings to escape the special html-characters with
  jecxz FoundSpecialHtmlChar

  loopEscapeStringList:
    push pebx
    call strlen
    lea pebx,pdword [pebx+peax+1] ;get next string in list
  loop loopEscapeStringList

FoundSpecialHtmlChar:
  pop pecx ;restore ECX

  xchg pedi,pesi ;swap current pos ((R/E)SI) with pointer unsend data ((R/E)DI)
  xchg pecx,pedx ;swap counter with size of unsend bytes
  ;--push peax ;save byte to escape

  ;call shortSendDataWithChunk or shortSendData
  if used is32
    call pdword [pebp+psize*8] ;send unsend data
  else
    call r12 ;send unsend data
  end if
  ;call shortSendData

  ;--pop peax ;restore byte to escape

  push '&' ;push % character
  mov pesi,pesp
  mov ecx,1 ;send three chars
  ;call shortSendDataWithChunk or shortSendData
  if used is32
    call pdword [pebp+psize*8] ;send the escaped char
  else
    call r12 ;send the escaped char
  end if
  ;call shortSendData ;send the escaped char
  add pesp,psize

  ;send escaped char
  push pebx
  call strlen
  mov pecx,peax
  mov pesi,pebx
  ;call shortSendDataWithChunk or shortSendData
  if used is32
    call pdword [pebp+psize*8] ;send the escaped char
  else
    call r12 ;send the escaped char
  end if
  ;call shortSendData ;send the escaped char

  push ';' ;push % character
  mov pesi,pesp
  mov ecx,1 ;send three chars
  ;call shortSendDataWithChunk or shortSendData
  if used is32
    call pdword [pebp+psize*8] ;send the escaped char
  else
    call r12 ;send the escaped char
  end if
  ;call shortSendData ;send the escaped char
  add pesp,psize

doNextloopThroughValue:
  xor ecx,ecx   ;zero unsend data
  xchg ecx,edx  ;restore counter
  mov pesi,pedi ;restore current pos

  dec pecx
  jnz loopThroughValue
  ;loop loopThroughValue
  jmp endloopThroughValue

nextloopThroughValue:
  inc edx ;one byte unsend
dec pecx
jnz loopThroughValue
;loop loopThroughValue

endloopThroughValue:
;send unsend data
mov pesi,pedi ;(R/E)SI =>with pointer unsend data
mov pecx,pedx ;(R/E)CX => size of unsend data

;call shortSendDataWithChunk or shortSendData
if used is32
  call pdword [pebp+psize*8] ;send unsend data
else
  call r12 ;send unsend data
end if
;call shortSendData ;send unsend data

mov peax,pdword [pebp+psize*7] ;pointer to EBP with buffer

mov ecx,dword [pebp-UsedSizeBydEnter-4]   ;Free size in buffer
mov dword [peax-UsedSizeBydEnter-4],ecx   ;Free size on buffer

mov ecx,dword [pebp-UsedSizeBydEnter-4*2] ;buffer size
mov dword [peax-UsedSizeBydEnter-4*2],ecx ;buffer size

mov pecx,pdword [pebp-UsedSizeBydEnter-psize-4*2]    ;buffer size
mov pdword [peax-UsedSizeBydEnter-psize-4*2],pecx    ;buffer

EndEscapeHtmlAndSendValue:
call dleave
if used is32
  ret psize*7
else
  ret psize*6
end if
;-------------------------------------------------------------------------

HtmlSpecialCharacters db ' "',"&'<>`",0
HtmlEscapedNames db "nbsp",0,"quot",0,"amp",0,"#39",0,"lt",0,"gt",0,"#96",0,"euro",0,"lsquo",0,"rsquo",0,"ldquo",0,"rdquo",0,"ndash",0,"mdash",0,"trade",0,"iexcl",0,"cent",0,"pound",0,"yen",0,"sect",0,"copy",0,"laquo",0,"reg",0,"`deg",0,"plusmn",0,"#180",0,"micro",0,"para",0,"middot",0,"raquo",0,"iquest",0,"Agrave",0,"Aacute",0,"Acirc",0,"Atilde",0,"Auml",0,"Aring",0,"AElig",0,"Ccedil",0,"Egrave",0,"Eacute",0,"Ecirc",0,"Euml",0,"Igrave",0,"Iacute",0,"Icirc",0,"Iuml",0,"Ntilde",0,"Ograve",0,"Oacute",0,"Ocirc",0,"Otilde",0,"Ouml",0,"Oslash",0,"Ugrave",0,"Uacute",0,"Ucirc",0,"Uuml",0,"szlig",0,"agrave",0,"aacute",0,"acirc",0,"atilde",0,"auml",0,"aring",0,"aelig",0,"ccedil",0,"egrave",0,"eacute",0,"ecirc",0,"euml",0,"igrave",0,"iacute",0,"icirc",0,"iuml",0,"ntilde",0,"ograve",0,"oacute",0,"ocirc",0,"otilde",0,"ouml",0,"divide",0,"oslash",0,"ugrave",0,"uacute",0,"ucirc",0,"uuml",0,"yuml",0

beginRespond db 'HTTP/1.1 '
endBeginRespond:

Statuscodes:
dw 100
db endCode100-code100
code100 db '100 Continue'
endCode100:

dw 200
db endCode200-code200
code200 db '200 OK'
endCode200:

dw 204
db endCode204-code204
code204 db '204 No Content'
endCode204:

dw 206
db endCode206-code206
code206 db '206 Partial Content'
endCode206:

dw 301
db endCode301-code301
code301 db '301 Moved Permanently'
endCode301:

dw 302
db endCode302-code302
code302 db '302 Found'
endCode302:

dw 304
db endCode304-code304
code304 db '304 Not Modified'
endCode304:

dw 307
db endCode307-code307
code307 db '307 Temporary Redirect'
endCode307:

dw 401
db endCode401-code401
code401 db '401 Unauthorized'
endCode401:

dw 403
db endCode403-code403
code403 db '403 Forbidden'
endCode403:

dw 404
db endCode404-code404
code404 db '404 Not Found'
endCode404:

dw 405
db endCode405-code405
code405 db '405 Am I supposed to know the answer to everything?' ;Method Not Allowed
endCode405:

dw 408
db endCode408-code408
code408 db '408 Took you long enough!' ;Request Time-out
endCode408:

dw 413
db endCode413-code413
code413 db '413 Just too much!' ;Request Entity Too Large
endCode413:

dw 500
db endCode500-code500
code500 db '500 Something is so wrong!' ;Internal Server Error
endCode500:

dw 0 ;end word
db 0 ;end byte '

contentTypeList:
db endUnknownBinary-unknownBinary
unknownBinary:
db 'unknown/binary'
endUnknownBinary:

db endTextPlain-textPlain
textPlain:
db 'text/plain'
endTextPlain:

db endTextHtml-textHtml
textHtml:
db 'text/html'
endTextHtml:

db endImagePng-imagePng
imagePng:
db 'image/png'
endImagePng:

db endImageGif-imageGif
imageGif:
db 'image/gif'
endImageGif:

db endImageJpeg-imageJpeg
imageJpeg:
db 'image/jpeg'
endImageJpeg:

db 0 ;end of content type list

newLine db 0x0D, 0x0A

dateName db 'Date: '
endDateName:

;serverNameAndContentType db "Server: TV's server/3 (Windows)",0x0D, 0x0A, "Content-Type: "
;endServerNameAndContentType:

serverName db "Server: ",HTTPHeaderServerName
endServerName:

contentType db 'Content-Type: '
endContentType:

contentLengthStr db 'Content-Length: '
endContentLengthStr:

contentRangeStr db 'Content-Range: bytes '
endContentRangeStr:

lastModifiedStr db 'Last-Modified: '
endLastModifiedStr:

expiresStr db 'Expires: '
endExpiresStr:

;Set-Cookie strings
setCookieStr db 'Set-Cookie: '
endSetCookieStr:

domainStr db '; Domain='
endDomainStr:

pathStr db '; Path='
endPathStr:

expireStr db '; Expires='
endExpireStr:

secureStr db '; Secure'
endSecureStr:

httpOnlyStr db '; HttpOnly'
endHttpOnlyStr:

;WWW-Authenticate strings
wwwAuthenticateStr db 'WWW-Authenticate: '
endwwwAuthenticateStr:

challengeBasic db 'Basic realm='
endChallengeBasic:

challengeDigest db 'Digest realm='
endChallengeDigest:

domainAuthStr db ', domain='
endDomainAuthStr:

nonceStr db ', nonce='
endNonceStr:

opaqueStr db ', opaque='
endOpaqueStr:

staleStr db ', stale='
endStaleStr:

trueStr db '"True"'
endTrueStr:

falseStr db '"False"'
endFalseStr:

algorithmStr db ', algorithm='
endAlgorithmStr:

qopStr db ', qop='
endQopStr:

qopAuthStr db '"auth"'
endQopAuthStr:

qopAuthIntStr db '"auth-int"'
endQopAuthIntStr:

connectionStr db 'Connection: '
endConnectionStr:

closeStr db 'close'
endCloseStr:

keepAliveStr db 'Keep-Alive'
endKeepAliveStr:

locationStr db 'Location: '
endLocationStr:

transferEncodingStr db 'Transfer-Encoding: chunked'
endTransferEncodingStr:

;mov pesi,pdword [pebx+pisze*2] ;respone_type
;;if response_type <= ammount of 'responseCommands' items, it is not a pointer
;cmp pesi,(endResponseCommands-responseCommands)/psize ;ammount of items in the responseCommands list
;ja respone_typeIsPointer
;
;mov pesi,pdword [responseCommands+pesi] ;get item from list
;
;respone_typeIsPointer:
;responseCommands:
;pointer _GET
;pointer _POST
;pointer _HEAD
;pointer _OPTIONS
;pointer _PUT
;pointer _DELETE
;pointer _TRACE
;pointer _CONNECT
;endResponseCommands:

;_GET       db 'GET',0
;_POST      db 'POST',0
;_HEAD      db 'HEAD',0
;_OPTIONS   db 'OPTIONS',0
;_PUT       db 'PUT',0
;_DELETE    db 'DELETE',0
;_TRACE     db 'TRACE',0
;_CONNECT   db 'CONNECT',0