
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

cp_directory db 'controlpanel\'
endCp_directory:

;------------------------------------------------------------------------------
;This function will handle the connection
;------------------------------------------------------------------------------
HandleConnection:
StartProc 2 ;Set arguments on stack
call denter

xor eax,eax ;return value

mov pedx,pdword [pebp+psize*2]  ;psess

cmp [psess_CONTROL_PANEL],pedx
jne endHandleConnection
;
;Handle Control Panel
;

if used is32
  sub pesp,8 ;make room for a long long
end if

;create size for a ResponseHeader structure
sub pesp,sizeOfResponseHeaderSize

clientLoop:
;zero structure
mov pedi,pesp
if used is32
  mov ecx,sizeOfResponseHeaderSize+8
else
  mov ecx,sizeOfResponseHeaderSize
end if
xor eax,eax
rep stosb

if used is32
  ;set contentLength
  mov dword [pesp+8+(psize*3)],esp
  add dword [pesp+8+(psize*3)],sizeOfResponseHeaderSize
end if

mov esi,0x1 ;GET_FROM_CLIENTSESSION
mov eax,CPO_NAME_MAX_TIMEOUT
call GetPluginDataInt ;(R/E)CX => maximum allowed timeout

;set members from the ResponseHeader structure
mov dword [pesp+4*2+psize],TEXT_HTML ;contentType

push 0

push pesp ;*error
push RPF_USE_TIME_LIMIT or RPF_USE_HEADER_SIZE_LIMIT or RPF_CHECK_VALID_METHOD ;flags
push pecx         ;maxTimeToReceive
push 1024*1024*5  ;maxHeaderSizeLimit (5MB)
push pdword [pebp+psize*3] ;ClientSession
push pdword [pebp+psize*2] ;ProtocolSession
call [RecvAndParseHTTP]
mov pebx,peax

pop peax ;error code

test pebx,pebx
jnz noErrorRecvHTTPHeader

  dec eax
  jz send413Error ;RPE_EXCEEDED_SIZE_LIMIT

  ;--dec eax
  ;--jz send408Error;RPE_EXCEEDED_TIME_LIMIT

  ;--dec eax
  ;--jz send405Error;RPE_UNKNOWN_METHOD

  ;--dec eax
  ;--jz send405Error;RPE_UNKNOWN_PROTOCOL

  ;--dec eax
  ;--jz endHandleConnection;RPE_CONNECTION_LOST
  jmp endHandleConnection

send413Error:
  ;set content length
  if used is32
    mov dword [pesp+4*2+psize*3],sizeContent413Error
  else
    mov qword [rsp+4*2+psize*3],endContent413Error-content413Error
  end if

  mov dword [pesp+8],413 ;set statusCode

  mov edi,endContent413Error-content413Error ;set size of content to send
  mov pesi,content413Error                   ;pointer to content to send
  jmp sendError

send404Error:
  ;set content length
  if used is32
    mov dword [pesp+4*2+psize*3],sizeContent404Error
  else
    mov qword [rsp+4*2+psize*3],endContent404Error-content404Error
  end if

  mov dword [pesp+8],404 ;set statusCode
  ;-or dword [pesp],FRH_CONNECTION_KEEP_ALIVE

  mov edi,endContent404Error-content404Error ;set size of content to send
  mov pesi,content404Error                   ;pointer to content to send
  jmp sendError

;--send408Error:
  ;--jmp endHandleConnection
  ;--;set content length
  ;--if used is32
  ;--  mov dword [esp+4*2+psize*3],sizeContent408Error
  ;--else
  ;--  mov qword [rsp+4*2+psize*3],endContent408Error-content413Error
  ;--end if

  ;--mov dword [pesp+8],408 ;set statusCode
  ;--xor dword [pesp],FRH_CONNECTION_KEEP_ALIVE ;delete flag

  ;--mov edi,endContent408Error-content408Error ;set size of content to send
  ;--mov pesi,content408Error                   ;pointer to content to send
  ;--jmp sendError

;--send405Error:
  ;jmp endHandleConnection
  ;set content length
  ;--if used is32
  ;--  mov dword [esp+4*2+psize*3],sizeContent405Error
  ;--else
  ;--  mov qword [rsp+4*2+psize*3],endContent405Error-content413Error
  ;--end if

  ;--mov dword [pesp+8],405 ;set statusCode
  ;--xor dword [pesp],FRH_CONNECTION_KEEP_ALIVE ;delete flag

  ;--mov edi,endContent405Error-content405Error ;set size of content to send
  ;--mov pesi,content405Error                   ;pointer to content to send
  ;--jmp sendError

sendWrongURLError: ;redirect this error to '/'
  mov dword [pesp+8],302 ;set statusCode
  mov pdword [pesp+4*2+9*psize+4*2+5*psize],baseLocation ;member location = '/'
  or dword [pesp],FRH_FORCE_ZERO_CONTENT_LENGTH ;add flag

  ;set content length
  if used is32
    mov dword [pesp+4*2+psize*3],zeroLength
  end if

  xor edi,edi ;zero size of content to send
  jmp sendError


sendError:
  ;send HTTP-header
  mov peax,pesp

  push DEFAULT_SENDTIMEOUT   ;timeout
  push peax                  ;ResponseHeader
  push pdword [pebp+psize*3] ;ClientSession
  push pdword [pebp+psize*2] ;ProtocolSession
  call [SendHTTPHeader]

  test pedi,pedi
  jz dontSendContent

  mov ecx,edi   ;size of buf
  call shortSendData

  dontSendContent:
  jmp closeHandleConnection


noErrorRecvHTTPHeader:
mov pecx,pdword [pebx] ;DefaultHTTPVariables
mov peax,pdword [pecx+13*psize] ;connection

;generate a hash of the value connection
push peax ;name of the variable
call GenerateHashCaseInsensitiveSafe
cmp eax,0xC2798F5C ;hash of string 'Keep-Alive'            0xDB9E4C25 = 'close'
jne isVersion1 ;set header-field in response header as 'close'

mov peax,pdword [pecx] ;pointer to httpVersion
mov eax,dword [peax]    ;get foure bytes from the version

cmp eax,'1.0'
je isVersion1 ;set header-field in response header as 'close', since 'Keep-alive' is not allowed in 1.0.

  mov esi,0x1 ;GET_FROM_CLIENTSESSION
  mov eax,CPO_NAME_KEEPALIVE
  call GetPluginDataInt ;(R/E)CX => boolean KEEPALIVE
  test ecx,ecx
  jz isVersion1

    or dword [pesp],FRH_CONNECTION_KEEP_ALIVE

isVersion1:
mov pedi,pdword [pebx+psize*5] ;URLbuffer

push 0
push pedi
call [ConvertEscapedCharactersToBytes]

push pedi
call [DeleteDoubleSlashesInUrl]

push pedi
call [DecodeURL]
test eax,eax
jz sendWrongURLError ;the url has to many parent directories / hack atempt

;calculate the size of the requested URL
push pedi
call strlen
add peax,[lenCurrentDirectory] ;add current drectory len
add peax,(endCp_directory-cp_directory) + (endIndexFile-startIndexFile) ;add size of cp_directory and index filename
inc peax ;increment for zero byte

;get enough memory to make a complete file URL
push peax
call GetMemLocation
xchg pedi,peax ;(R/E)DI => pointer to file URL, (R/E)AX => pointer to URL
mov pedx,pedi  ;(R/E)DI => pointer to file URL

mov pesi,[currentDirectory]
mov pecx,[lenCurrentDirectory]
rep movsb ;copy current directory

mov pesi,cp_directory
mov ecx,endCp_directory-cp_directory
rep movsb ;copy the cp_directory


mov pesi,peax ;(R/E)SI => pointer to URL
push pesi
call strlen
mov pecx,peax
rep movsb ;copy the URL

cmp byte[pedi-1],'\'
je copyIndexFilename

cmp byte[pedi-1],'/'
jne dontCopyIndexFilename

copyIndexFilename:

  ;copy filename to URL
  mov pesi,indexFile
  mov ecx,endIndexFile-startIndexFile
  rep movsb ;copy the cp_directory

  mov pdword [pesp+8+(psize*2)],indexFileExtension ;set contentTypeExtension
  or dword [pesp],FRH_USE_CONTENT_TYPE_EXTENSION ;add flag
dontCopyIndexFilename:

;open a handle to the file
push 0 ;flag => READ
push pedx
call openFileHandle
mov pesi,peax ;(R/E)SI => file handle

;free the complete URL
push pedx
call FreeMemLocation

mov peax,pesi
inc peax ;failed handle is -1, make it 0
test peax,peax ;test on failed handle
je send404Error

;search for the extension
mov pedi,pdword [pebx+psize*5] ;URLbuffer
push pedi
call strlen
add pedi,peax  ;Get to the end of the request URL
mov pecx,peax  ;ECX => size of the string

;Check if the extension is already filled
cmp pdword [pesp+8+(psize*2)],0
jne notFoundExtension ;jump if extension is already filled

;Start seacrhing for the extension
std ;Set search direction backwards
mov al,'.' ;search for a dot
repnz scasb ;start searching
cld ;Set search direction forward
jnz notFoundExtension

  inc pedi
  mov pdword [pesp+8+(psize*2)],pedi ;set contentTypeExtension
  or dword [pesp],FRH_USE_CONTENT_TYPE_EXTENSION ;add flag

notFoundExtension:
mov dword [pesp+8],200 ;set statusCode


;lock linked-list with internal protocol structures
call GetInternalProtocolstructureReadLock

;get file size of the file
push pesi
call getfilesize
push peax ;save for later use
push peax ;push for readfile

inc peax ;when the page needs to be parsed, we need a zero byte at the end!

;get memory to request read this file to
push peax
call GetMemLocation
mov pedx,peax ;(R/E)DX => pointer for read file


;read the file
;push pedi ;size of file to read
push pedx ;buffer
push pesi ;file handle
call readfile
;close file handle
push pesi ;file handle
call closefile

;--use content_type_extension to send the contenttype
mov peax,pdword [pesp+psize+8+(psize*2)] ;contentTypeExtension
call [GetMimeTypeFromExtension]
test peax,peax
cmovz ecx,eax ;ECX => zero
jz noMimType

  ;get first four bytes
  mov ecx,dword [peax]

  ;free the mim-type
  push peax

  mov peax,[FreeMimeTypeFromExtension]
  test peax,peax
  jnz callFreeMimeTypeFromExtension

     call FreeMemLocation
     jmp noMimType

callFreeMimeTypeFromExtension:
  call [FreeMimeTypeFromExtension]



noMimType:


pop peax ;(R/E)AX => size of the buffer
mov pedi,pedx ;(R/E)DI => pointer to buffer

cmp ecx,'text'
cmovne pecx,peax ;if not TEXT, (R/E)CX => size of buffer
cmovne peax,pedi ;if not TEXT, (R/E)AX => pointer to buffer
jne sendDataNotText


;TODO: delete?
;test dword [pesp],FRH_CONNECTION_KEEP_ALIVE
;jz dontDeleteKeepAliveFlag

;  xor dword [pesp],FRH_CONNECTION_KEEP_ALIVE ;delete flag

;dontDeleteKeepAliveFlag:

;We are going to send chunks and the end of the header will be send with the first chunk
or dword [pesp],FRH_TRANSFER_ENCODING_CHUNKED OR FRH_DONT_END_HEADER

mov pesi,pedx ;(R/E)SI => data buffer

;send HTTP-header
mov peax,pesp

push DEFAULT_SENDTIMEOUT   ;timeout
push peax                  ;ResponseHeader
push pdword [pebp+psize*3] ;ClientSession
push pdword [pebp+psize*2] ;ProtocolSession
call [SendHTTPHeader]

;remove flags for next request
xor dword [pesp],FRH_TRANSFER_ENCODING_CHUNKED OR FRH_DONT_END_HEADER


;replace tags with simple bytes
push pesi ;buffer
call PreParseElement

;replace the tags with ectual data and send the data
push pesi                  ;parsed page
push pebx                  ;HTTPSession
push pdword [pebp+psize*3] ;listensession
push pdword [pebp+psize*2] ;protocolsession
call ParseAndSend

push pesi
push pecx

mov pesi,nextChunkstr
mov ecx,endNextChunkstr-nextChunkstr
call shortSendData ;send (R/E)SI with size ECX

pop pecx
pop pesi

jmp succesEndHandleConnection

sendDataNotText:
push peax ;save data to send

if used is32
  ;set contentLength
  mov eax,dword [pesp+psize+8+(psize*3)]
  mov dword [peax],ecx
else
  mov qword [rsp+psize+8+(psize*3)],rcx
end if

lea peax,pdword [pesp+psize]

;send HTTP-header
push DEFAULT_SENDTIMEOUT   ;timeout
push peax                  ;ResponseHeader  (already pushed)
push pdword [pebp+psize*3] ;ClientSession
push pdword [pebp+psize*2] ;ProtocolSession
call [SendHTTPHeader]

;send file
pop pesi
call shortSendData
;-------------

succesEndHandleConnection:
  ;free the send file
  push pesi
  call FreeMemLocation

closeHandleConnection:
test pebx,pebx
jz dontReleaseHTTPSession

  push pebx
  call [FreeHTTPSession]

dontReleaseHTTPSession:
test dword [pesp],FRH_CONNECTION_KEEP_ALIVE
jnz clientLoop

endHandleConnection:

call dleave
EndProc 2
ret (FunctionReturnFactor*2)

;------------------------------------------------------------------------------
;This function will send data in a chunk
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)SI => buffer
;    (R/E)CX => size
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendDataWithChunk:
push pedi
push pecx

sub pesp,0x20
mov pedi,pesp

;begin chunk with new line
mov byte [pedi],0x0D
mov byte [pedi+1],0x0A
add pedi,2

;next convert chunk size to string
push 0x10                 ;base at 16
push pedi                 ;output string
push pecx                 ;value
call IntToStr

;add another new line (for the data
push pedi
call strlen
mov byte [pedi+peax],0x0D
mov byte [pedi+peax+1],0x0A

sub pedi,2 ;set pointer to first new line
add peax,4 ;add lines to len

push peax

push pesp ;len
push pedi ;buf
push DEFAULT_SENDTIMEOUT    ;timeout
push RSF_RETURN_ON_COMPLETE  ;flags
push pdword [pebp+psize*3] ;ClientSession
push pdword [pebp+psize*2] ;ProtocolSession
call [SendData]

add pesp,0x20+psize ;buff and size of stack

;restore registers
pop pecx
pop pedi

call shortSendData ;send the data
ret

;------------------------------------------------------------------------------
;This function will send data in a chunk
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)SI => buffer
;    (R/E)DX => size
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendDataEDXWithChunk:
push pecx

mov pecx,pedx
call shortSendDataWithChunk

pop pecx
ret

;------------------------------------------------------------------------------
;This function will send data
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)SI => buffer
;    (R/E)CX => size
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendData:
push pecx

push pesp ;len
push pesi ;buf
push DEFAULT_SENDTIMEOUT    ;timeout
push RSF_RETURN_ON_COMPLETE  ;flags
push pdword [pebp+psize*3] ;ClientSession
push pdword [pebp+psize*2] ;ProtocolSession
call [SendData]

add pesp,psize
test eax,eax
ret

;------------------------------------------------------------------------------
;This function will send data
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)AX => buffer
;    (R/E)CX => size
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendDataEAX:
push pesi

mov pesi,peax
call shortSendData

pop pesi
ret

;------------------------------------------------------------------------------
;This function will send data
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)SI => buffer
;    (R/E)DX => size
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendDataEDX:
push pecx

mov pecx,pedx
call shortSendData

pop pecx
ret

shortSendStringWithChunk:
push pebx
xor ebx,ebx
inc ebx
jmp shortSendStringEBXSet

;------------------------------------------------------------------------------
;This function will send an ASCII string
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)AX => string
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendString:
push pebx
xor ebx,ebx

shortSendStringEBXSet:
push pecx
push pesi

mov pesi,peax

push pesi
call strlen
mov pecx,peax

test ebx,ebx
jz dontSendWithChunk3

  call shortSendDataWithChunk
  jmp endShortSendString

dontSendWithChunk3:
  call shortSendData

endShortSendString:
pop pesi
pop pecx
pop pebx
ret

;------------------------------------------------------------------------------
;This function will send an ASCII string (but first escapes the special HTML-characters) in chunks
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)AX => string
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
if used shortSendStringEscapeWithChunks
shortSendStringEscapeWithChunks:
  push EHSV_SEND_IN_CHUNKS   ;flags
  push DEFAULT_SENDTIMEOUT   ;timeout
  push peax                  ;string
  push pdword [pebp+psize*3] ;ClientSession
  push pdword [pebp+psize*2] ;ProtocolSession
  call [EscapeHtmlAndSendValue]
ret
end if

;------------------------------------------------------------------------------
;This function will send an ASCII string (but first escapes the special HTML-characters)
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)AX => string
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
if used shortSendStringEscape
shortSendStringEscape:
  push 0                     ;flags
  push DEFAULT_SENDTIMEOUT   ;timeout
  push peax                  ;string
  push pdword [pebp+psize*3] ;ClientSession
  push pdword [pebp+psize*2] ;ProtocolSession
  call [EscapeHtmlAndSendValue]
ret
end if

;------------------------------------------------------------------------------
;This function will find an InteralProtocolStructure by a ProtocolSession
;input:
;1: ProtocolSession
;output:
;    (R/E)AX => pointer to a InteralProtocolStructure
;------------------------------------------------------------------------------
GetInternalProtocolStructure:
call denter

call GetInternalProtocolstructureReadLock

mov pedi,[cpyInternalProtocolstructure];(R/E)DI => pointer to first InternalProtocolStructure

loopThroughInternalProtocolStructureLinkedList:
  mov pesi,pedi ;(R/E)SI => pointer to current InternalProtocolStructure

  lodsp ;het pointer to the next InternalProtocolStructure
  mov pedi,peax ;(R/E)DI => next protocol structure

  lodsp ;get psess
  cmp peax,pdword [pebp+psize*2] ;is this the protocol we are looking for
  je foundPsess

  test pedi,pedi ;test is next InternalProtocolStructure is zero
jnz loopThroughInternalProtocolStructureLinkedList

xor esi,esi ;clear (R/E)SI
jmp endGetInternalProtocolStructure

foundPsess:
sub pesi,psize*2  ;substract by two members, since this was incremented buy the lods(d/q) calls

endGetInternalProtocolStructure:
call ReleaseInternalProtocolstructureReadLock

mov peax,pesi

call dleave
ret psize

;------------------------------------------------------------------------------
;This function will get an read lock on the cpyInternalProtocolstructure linked list
;input:
;  none
;output:
;    none
;------------------------------------------------------------------------------
GetInternalProtocolstructureReadLock:
call denter

jmp FirstTyrReadLock

TryReadLockAgain:
  push 1
  call sleep

FirstTyrReadLock:
  if used is32
    ;EDX(high) EAX(low) => Qword to compair [cpyInternalProtocolstructureSignal] with
    xor edx,edx
    mov eax,[reading]

    ;ecx(high) ebx(low) => Qword to set [cpyInternalProtocolstructureSignal] to
    mov ebx,eax
    inc ebx ;increment the reading threads
    xor ecx,ecx
    lock cmpxchg8b [cpyInternalProtocolstructureSignal]
  else
    mov eax,[reading]   ;RAX => Qword to compair [cpyInternalProtocolstructureSignal] with
    mov ecx,eax         ;RCX => Qword to check [cpyInternalProtocolstructureSignal] to
    inc ecx             ;increment the ammount of 'reading' threads

    lock cmpxchg [cpyInternalProtocolstructureSignal], rcx
  end if
jnz TryReadLockAgain

call dleave
ret

;------------------------------------------------------------------------------
;This function will release the earlier gathered read lock on the cpyInternalProtocolstructure linked list
;input:
;  none
;output:
;    none
;------------------------------------------------------------------------------
ReleaseInternalProtocolstructureReadLock:
call denter

TryReleaseReadLockAgain:
  if used is32
    ;EDX(high) EAX(low) => Qword to compair [cpyInternalProtocolstructureSignal] with
    xor edx,edx
    mov eax,[reading]

    ;ecx(high) ebx(low) => Qword to set [cpyInternalProtocolstructureSignal] to
    mov ebx,eax
    dec ebx ;decrement the reading threads
    xor ecx,ecx ;set write lock
    lock cmpxchg8b [cpyInternalProtocolstructureSignal]
  else
    mov eax,[reading]   ;RAX => Qword to compair [cpyInternalProtocolstructureSignal] with
    mov ecx,eax         ;RCX => Qword to check [cpyInternalProtocolstructureSignal] to
    dec ecx             ;increment the ammount of 'reading' threads

    lock cmpxchg [cpyInternalProtocolstructureSignal], rcx
  end if
jnz TryReleaseReadLockAgain

call dleave
ret

if used is32
  zeroLength dq 0
end if
baseLocation db '/',0

;--content200OK1 db "<HTML><HEAD><TITLE>200 OK</TITLE></HEAD><BODY><H1>Success</H1>We successfully talked the HTTP-protocol. The URL you requested is:"
;--endContent200OK1:
;--content200OK2 db "<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
;--endContent200OK2:

;if 32 bit, create an qword containing the size of the error
if used is32
  sizeContent413Error dq endContent413Error-content413Error
end if
content413Error db '<!DOCTYPE HTML><HTML><HEAD><meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1">', "<TITLE>413 Just too much!</TITLE></HEAD><BODY><H1>Less is more</H1>Your client just send too much data, keep your request clean and easy and try again.<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
endContent413Error:

;if 32 bit, create an qword containing the size of the error
if used is32
  sizeContent404Error dq endContent404Error-content404Error
end if
content404Error db '<!DOCTYPE HTML><HTML><HEAD><meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1">', "<TITLE>404 Not Found</TITLE></HEAD><BODY><H1>My apology!</H1>The file you were looking for is unfortunately not found on this server.<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
endContent404Error:

startIndexFile:
  indexFile db 'index'
  indexFileExtension db '.htm',0
endIndexFile:

nextChunkstr db 0x0D, 0x0A, 0x30,0x0D, 0x0A,0x0D, 0x0A
endNextChunkstr:

;--;if 32 bit, create an qword containing the size of the error
;--if used is32
;--  sizeContent408Error dq endContent408Error-content408Error
;--end if
;--content408Error db "<HTML><HEAD><TITLE>408 Took you long enough!</TITLE></HEAD><BODY><H1>Time is money</H1>It took too long for your client to transmit the request to the server. Time is money, so please speed up and try again.<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
;--uendContent408Error:

;-;if 32 bit, create an qword containing the size of the error
;-if used is32
;-  sizeContent405Error dq endContent405Error-content405Error
;-end if
;-content405Error db "<HTML><HEAD><TITLE>405 Am I supposed to know the answer to everything?</TITLE></HEAD><BODY><H1>Can you reformulate</H1>The server is unable to understand the question you trying to ask. Can you please reformulate your question and try again?<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
;-endContent405Error: